package com.loit.seata.order.example.web.controller;

import com.loit.build.spi.common.dto.BaseResponse;
import com.loit.build.spi.common.exception.BusinessException;
import com.loit.seata.order.example.service.biz.OrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@RestController
@RequestMapping("order")
@RefreshScope
@Api(tags = "订单管理", value = "OrderController")
@Slf4j
public class OrderController {

    @Resource
    private OrderService orderService;

    @Value("${user.name}")
    private String userName;

    @Value("${user.age}")
    private int age;

    @RequestMapping("/user")
    public String simple() {
        return "userName: " + userName + "age:" + age;
    }

    /**
     * 下单：插入订单表、扣减库存, 成功下单
     *
     * @return
     */
    @ApiOperation(value = "成功下单", notes = "下单：插入订单表、扣减库存, 成功下单")
    @GetMapping("/placeOrder/commit")
    public BaseResponse placeOrderCommit() {

        try {
            orderService.placeOrder("1", "product-1", 1);
        } catch (Exception e) {
            log.error("", e);
            throw new BusinessException("下单失败,请稍后再试");
        }

        return BaseResponse.ofSuccess(true);
    }

    /**
     * 下单：插入订单表、扣减库存，模拟回滚
     *
     * @return
     */
    @ApiOperation(value = "模拟回滚", notes = "下单：插入订单表、扣减库存，模拟回滚")
    @GetMapping("/placeOrder/rollback")
    public BaseResponse placeOrderRollback() {
        // product-2 扣库存时模拟了一个业务异常,
        orderService.placeOrder("1", "product-2", 1);
        return BaseResponse.ofSuccess(true);
    }

    @PostMapping("/placeOrder")
    @ApiOperation(value = "下单(传参数)", notes = "插入订单表、扣减库存，模拟回滚, 传参数")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", name = "userId", value = "用户ID", dataType = "String"),
            @ApiImplicitParam(paramType = "query", name = "commodityCode", value = "商品code", dataType = "String"),
            @ApiImplicitParam(paramType = "query", name = "count", value = "购买数量", dataType = "Integer")
    })
    public BaseResponse placeOrder(String userId, String commodityCode, Integer count) {
        orderService.placeOrder(userId, commodityCode, count);
        return BaseResponse.ofSuccess(true);
    }


    @GetMapping("/echo")
    @ApiOperation(value = "测试接口响应")
    public BaseResponse echo() {
        return BaseResponse.ofSuccess(true);
    }
}
